//
// general_support.h
//

#ifndef     _GENERAL_SUPPORT
#   define  _GENERAL_SUPPORT

#include "double_star.h"
#include "star_support.h"

void put_state(double_star*, star_state&, star_state&);
void put_initial_conditions(double_init& );

#endif		// _GENERAL_SUPPORT
